#indexMarket.py
from enum import Enum

class indexMarket(Enum):
 SET     = "SET"
 SET50   = "SET50"
 SET100  = "SET100"
 sSET    = "sSET"
 SETCLMV = "SETCLMV"
 SETHD   = "SETHD"
 SETTHSI = "SETTHSI"
 SETWB   = "SETWB"
 mai     = "mai" 